==> klayout-0.10.tar.gz: klayout-0.10/src/CellSelectionFormImpl.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <algorithm>
#include <string>
#include <set>

#include "CellSelectionFormImpl.h"
#include "dbCellInst.h"

#include "tlException.h"
#include "tlAlgorithm.h"

#include "layCellTreeModel.h"
#include "layLayoutView.h"

#include <QtGui/QHeaderView>


// ------------------------------------------------------------

CellSelectionFormImpl::CellSelectionFormImpl (QWidget *parent, lay::LayoutView *view, const char *name, Qt::WFlags fl)
  : QDialog (parent, name, fl), Ui::CellSelectionForm (),
    mp_view (view),
    m_current_cv (-1),
    m_name_cb_enabled (true),
    m_cells_cb_enabled (true),
    m_children_cb_enabled (true),
    m_parents_cb_enabled (true)
{
  Ui::CellSelectionForm::setupUi (this);

  // signals and slots connections
  connect (cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  connect (cb_views, SIGNAL(activated(int)), this, SLOT(view_changed(int)));
  connect (tb_set_parent, SIGNAL(clicked()), this, SLOT(set_parent()));
  connect (tb_set_child, SIGNAL(clicked()), this, SLOT(set_child()));
  connect (le_cell_name, SIGNAL(textChanged(const QString&)), this, SLOT(name_changed(const QString&)));
  connect (ok_button, SIGNAL(clicked()), this, SLOT(accept()));

  connect (lv_parents, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(parent_changed(const QModelIndex &)));
  connect (lv_children, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(child_changed(const QModelIndex &)));

  m_cellviews.reserve (mp_view->cellviews ());
  for (unsigned int i = 0; i < mp_view->cellviews (); ++i) {
    m_cellviews.push_back (mp_view->cellview (i));
  }

  apply_button->hide ();

  if (! m_cellviews.empty ()) {

    m_current_cv = view->active_cellview_index ();

    int cvi = 0;
    for (std::vector<lay::CellView>::const_iterator cv = m_cellviews.begin (); cv != m_cellviews.end (); ++cv, ++cvi) {
      cb_views->insertItem (((*cv)->name () + " (@" + tl::to_string (cvi + 1) + ")").c_str ());
    }
    cb_views->setCurrentItem (m_current_cv);

    if (m_cellviews.size () == 1) {
      cb_views->hide ();
      layout_lbl->hide ();
    } else {
      cb_views->show ();
      layout_lbl->show ();
    }

    lv_cells->header ()->hide ();
    lv_cells->setRootIsDecorated (false);

    lv_children->header ()->hide ();
    lv_children->setRootIsDecorated (false);

    lv_parents->header ()->hide ();
    lv_parents->setRootIsDecorated (false);
      
    update_cell_list ();  

  }

}

void 
CellSelectionFormImpl::update_cell_list () 
{
  if (m_current_cv < 0 || m_current_cv >= int (m_cellviews.size ())) {
    return;
  }

  if (lv_cells->model ()) {
    delete lv_cells->model ();
  }

  lay::CellTreeModel *model = new lay::CellTreeModel (lv_cells, mp_view, m_current_cv, lay::CellTreeModel::Flat);
  
  lv_cells->setModel (model);
  //  connect can only happen after setModel()
  connect (lv_cells->selectionModel (), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(cell_changed(const QModelIndex &, const QModelIndex &)));

  if (! m_cellviews [m_current_cv].path ().empty ()) {
    select_entry (m_cellviews [m_current_cv].path ().back ());
  }
}

void 
CellSelectionFormImpl::update_parents_list ()
{
  m_parents_cb_enabled = false;

  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    if (lv_parents->model ()) {
      delete lv_parents->model ();
    }
    lv_parents->setModel (new lay::CellTreeModel (lv_parents, mp_view, m_current_cv, lay::CellTreeModel::Flat | lay::CellTreeModel::Parents, model->cell (lv_cells->selectionModel ()->currentIndex ())));

  }

  m_parents_cb_enabled = true;
}

void 
CellSelectionFormImpl::update_children_list ()
{
  m_children_cb_enabled = false;
  
  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    if (lv_children->model ()) {
      delete lv_children->model ();
    }
    lv_children->setModel (new lay::CellTreeModel (lv_children, mp_view, m_current_cv, lay::CellTreeModel::Flat | lay::CellTreeModel::Children, model->cell (lv_cells->selectionModel ()->currentIndex ())));

  }

  m_children_cb_enabled = true;
}

int 
CellSelectionFormImpl::selected_cellview_index () const
{
  return m_current_cv;
}

const lay::CellView &
CellSelectionFormImpl::selected_cellview () const
{
  tl_assert (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ()));
  return m_cellviews [m_current_cv];
}

void 
CellSelectionFormImpl::commit_cv ()
{
  //  update the cell view 
  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    const db::Cell *cell = model->cell (lv_cells->selectionModel ()->currentIndex ());
    if (cell) {
      m_cellviews [m_current_cv].set_cell (cell->cell_index ());
    }

  }
}

void 
CellSelectionFormImpl::view_changed (int cv)
{
  commit_cv ();
  m_current_cv = cv;
  update_cell_list ();  
}

void
CellSelectionFormImpl::accept ()
{
  commit_cv ();
  QDialog::accept ();
}

void 
CellSelectionFormImpl::cell_changed (const QModelIndex &current, const QModelIndex &)
{
  if (m_cells_cb_enabled) {

    m_name_cb_enabled = false;

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());
    if (model) {
      le_cell_name->setText (model->cell_name (current));
    }

    m_name_cb_enabled = true;

    update_children_list ();
    update_parents_list ();

  }
}

void
CellSelectionFormImpl::set_child ()
{
  child_changed (lv_children->selectionModel ()->currentIndex ());
}

void 
CellSelectionFormImpl::child_changed(const QModelIndex &current)
{
  if (m_children_cb_enabled && current.isValid ()) {
    if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {
      lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_children->model ());
      select_entry (model->cell_index (lv_children->selectionModel ()->currentIndex ()));
    }
  }
}

void
CellSelectionFormImpl::set_parent ()
{
  parent_changed (lv_parents->selectionModel ()->currentIndex ());
}

void 
CellSelectionFormImpl::parent_changed(const QModelIndex &current)
{
  if (m_parents_cb_enabled && current.isValid ()) {
    if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {
      lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_parents->model ());
      select_entry (model->cell_index (lv_parents->selectionModel ()->currentIndex ()));
    }
  }
}

void 
CellSelectionFormImpl::select_entry (lay::CellView::cell_index_type ci)
{
  m_cells_cb_enabled = false;

  lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

  //  select the current entry
  QModelIndex mi;
  for (int c = 0; c < model->toplevel_items (); ++c) {
    lay::CellTreeItem *item = model->toplevel_item (c);
    if (item->cell_index () == ci) {
      mi = model->model_index (item);
      break;
    }
  }
        
  if (mi.isValid ()) {

    m_cells_cb_enabled = false;
    lv_cells->selectionModel ()->setCurrentIndex (mi, QItemSelectionModel::Clear | QItemSelectionModel::SelectCurrent);
    lv_cells->scrollTo (mi);
    m_cells_cb_enabled = true;

    m_name_cb_enabled = false;
    le_cell_name->setText (model->cell_name (mi));
    m_name_cb_enabled = true;

    //  do child list updates in a user event handler. Otherwise changing the models
    //  immediately interferes with Qt's internal logic.
    QCoreApplication::instance ()->postEvent (this, new QEvent (QEvent::User));

  }
  
  m_cells_cb_enabled = true;
}

void 
CellSelectionFormImpl::customEvent (QEvent *event)
{
  update_children_list ();
  update_parents_list ();
}

void 
CellSelectionFormImpl::name_changed (const QString &s)
{
  if (m_name_cb_enabled) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    QModelIndex mi = model->locate (s.ascii ());
    if (mi.isValid ()) {

      m_cells_cb_enabled = false;
      lv_cells->selectionModel ()->setCurrentIndex (mi, QItemSelectionModel::SelectCurrent);
      lv_cells->scrollTo (mi);
      update_children_list ();
      update_parents_list ();
      m_cells_cb_enabled = true;

    }

  }
}

==> klayout-0.14.tar.gz: klayout-0.14/src/CellSelectionFormImpl.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include <algorithm>
#include <string>
#include <set>

#include "CellSelectionFormImpl.h"
#include "dbCellInst.h"

#include "tlException.h"
#include "tlAlgorithm.h"
#include "tlAssert.h"

#include "layCellTreeModel.h"
#include "layLayoutView.h"

#include <QtGui/QHeaderView>


// ------------------------------------------------------------

CellSelectionFormImpl::CellSelectionFormImpl (QWidget *parent, lay::LayoutView *view, const char *name, Qt::WFlags fl)
  : QDialog (parent, name, fl), Ui::CellSelectionForm (),
    mp_view (view),
    m_current_cv (-1),
    m_name_cb_enabled (true),
    m_cells_cb_enabled (true),
    m_children_cb_enabled (true),
    m_parents_cb_enabled (true),
    m_update_all_dm (this, &CellSelectionFormImpl::update_all)
{
  Ui::CellSelectionForm::setupUi (this);

  // signals and slots connections
  connect (cancel_button, SIGNAL(clicked()), this, SLOT(reject()));
  connect (cb_views, SIGNAL(activated(int)), this, SLOT(view_changed(int)));
  connect (tb_set_parent, SIGNAL(clicked()), this, SLOT(set_parent()));
  connect (tb_set_child, SIGNAL(clicked()), this, SLOT(set_child()));
  connect (pb_hide, SIGNAL(clicked()), this, SLOT(hide_cell()));
  connect (pb_show, SIGNAL(clicked()), this, SLOT(show_cell()));
  connect (le_cell_name, SIGNAL(textChanged(const QString&)), this, SLOT(name_changed(const QString&)));
  connect (ok_button, SIGNAL(clicked()), this, SLOT(accept()));

  connect (lv_parents, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(parent_changed(const QModelIndex &)));
  connect (lv_children, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(child_changed(const QModelIndex &)));

  m_cellviews.reserve (mp_view->cellviews ());
  for (unsigned int i = 0; i < mp_view->cellviews (); ++i) {
    m_cellviews.push_back (mp_view->cellview (i));
  }

  apply_button->hide ();

  if (! m_cellviews.empty ()) {

    m_current_cv = view->active_cellview_index ();

    int cvi = 0;
    for (std::vector<lay::CellView>::const_iterator cv = m_cellviews.begin (); cv != m_cellviews.end (); ++cv, ++cvi) {
      cb_views->insertItem (((*cv)->name () + " (@" + tl::to_string (cvi + 1) + ")").c_str ());
    }
    cb_views->setCurrentItem (m_current_cv);

    if (m_cellviews.size () == 1) {
      cb_views->hide ();
      layout_lbl->hide ();
    } else {
      cb_views->show ();
      layout_lbl->show ();
    }

    lv_cells->header ()->hide ();
    lv_cells->setRootIsDecorated (false);

    lv_children->header ()->hide ();
    lv_children->setRootIsDecorated (false);

    lv_parents->header ()->hide ();
    lv_parents->setRootIsDecorated (false);
      
    update_cell_list ();  

  }

}

void 
CellSelectionFormImpl::update_cell_list () 
{
  if (m_current_cv < 0 || m_current_cv >= int (m_cellviews.size ())) {
    return;
  }

  if (lv_cells->model ()) {
    delete lv_cells->model ();
  }

  lay::CellTreeModel *model = new lay::CellTreeModel (lv_cells, mp_view, m_current_cv, lay::CellTreeModel::Flat);
  
  lv_cells->setModel (model);
  //  connect can only happen after setModel()
  connect (lv_cells->selectionModel (), SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)), this, SLOT(cell_changed(const QModelIndex &, const QModelIndex &)));

  lay::CellView::unspecific_cell_path_type path (m_cellviews [m_current_cv].combined_unspecific_path ());
  if (! path.empty ()) {
    select_entry (path.back ());
  }
}

void 
CellSelectionFormImpl::update_parents_list ()
{
  m_parents_cb_enabled = false;

  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    if (lv_parents->model ()) {
      delete lv_parents->model ();
    }
    lv_parents->setModel (new lay::CellTreeModel (lv_parents, mp_view, m_current_cv, lay::CellTreeModel::Flat | lay::CellTreeModel::Parents, model->cell (lv_cells->selectionModel ()->currentIndex ())));

  }

  m_parents_cb_enabled = true;
}

void 
CellSelectionFormImpl::update_children_list ()
{
  m_children_cb_enabled = false;
  
  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    if (lv_children->model ()) {
      delete lv_children->model ();
    }
    lv_children->setModel (new lay::CellTreeModel (lv_children, mp_view, m_current_cv, lay::CellTreeModel::Flat | lay::CellTreeModel::Children, model->cell (lv_cells->selectionModel ()->currentIndex ())));

  }

  m_children_cb_enabled = true;
}

int 
CellSelectionFormImpl::selected_cellview_index () const
{
  return m_current_cv;
}

const lay::CellView &
CellSelectionFormImpl::selected_cellview () const
{
  tl_assert (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ()));
  return m_cellviews [m_current_cv];
}

void 
CellSelectionFormImpl::commit_cv ()
{
  //  update the cell view 
  if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    const db::Cell *cell = model->cell (lv_cells->selectionModel ()->currentIndex ());
    if (cell) {
      m_cellviews [m_current_cv].set_cell (cell->cell_index ());
    }

  }
}

void 
CellSelectionFormImpl::view_changed (int cv)
{
  commit_cv ();
  m_current_cv = cv;
  update_cell_list ();  
}

void
CellSelectionFormImpl::accept ()
{
  commit_cv ();
  QDialog::accept ();
}

void 
CellSelectionFormImpl::cell_changed (const QModelIndex &current, const QModelIndex &)
{
  if (m_cells_cb_enabled) {

    m_name_cb_enabled = false;

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());
    if (model) {
      le_cell_name->setText (model->cell_name (current));
    }

    m_name_cb_enabled = true;

    update_children_list ();
    update_parents_list ();

  }
}

void
CellSelectionFormImpl::set_child ()
{
  child_changed (lv_children->selectionModel ()->currentIndex ());
}

void 
CellSelectionFormImpl::child_changed(const QModelIndex &current)
{
  if (m_children_cb_enabled && current.isValid ()) {
    if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {
      lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_children->model ());
      select_entry (model->cell_index (lv_children->selectionModel ()->currentIndex ()));
    }
  }
}

void
CellSelectionFormImpl::set_parent ()
{
  parent_changed (lv_parents->selectionModel ()->currentIndex ());
}

void 
CellSelectionFormImpl::parent_changed(const QModelIndex &current)
{
  if (m_parents_cb_enabled && current.isValid ()) {
    if (m_current_cv >= 0 && m_current_cv < int (m_cellviews.size ())) {
      lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_parents->model ());
      select_entry (model->cell_index (lv_parents->selectionModel ()->currentIndex ()));
    }
  }
}

void 
CellSelectionFormImpl::select_entry (lay::CellView::cell_index_type ci)
{
  m_cells_cb_enabled = false;

  lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

  //  select the current entry
  QModelIndex mi;
  for (int c = 0; c < model->toplevel_items (); ++c) {
    lay::CellTreeItem *item = model->toplevel_item (c);
    if (item->cell_index () == ci) {
      mi = model->model_index (item);
      break;
    }
  }
        
  if (mi.isValid ()) {

    m_cells_cb_enabled = false;
    lv_cells->selectionModel ()->setCurrentIndex (mi, QItemSelectionModel::Clear | QItemSelectionModel::SelectCurrent);
    lv_cells->scrollTo (mi);
    m_cells_cb_enabled = true;

    m_name_cb_enabled = false;
    le_cell_name->setText (model->cell_name (mi));
    m_name_cb_enabled = true;

    //  do child list updates in a user event handler. Otherwise changing the models
    //  immediately interferes with Qt's internal logic. So we do an deferred update.
    m_update_all_dm ();

  }
  
  m_cells_cb_enabled = true;
}

void 
CellSelectionFormImpl::update_all ()
{
  update_children_list ();
  update_parents_list ();
}

void 
CellSelectionFormImpl::name_changed (const QString &s)
{
  if (m_name_cb_enabled) {

    lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());

    QModelIndex mi = model->locate (s.ascii ());
    if (mi.isValid ()) {

      m_cells_cb_enabled = false;
      lv_cells->selectionModel ()->setCurrentIndex (mi, QItemSelectionModel::SelectCurrent);
      lv_cells->scrollTo (mi);
      update_children_list ();
      update_parents_list ();
      m_cells_cb_enabled = true;

    }

  }
}

void
CellSelectionFormImpl::show_cell ()
{
  lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());
  if (! model) {
    return;
  }
  if (m_current_cv < 0 || m_current_cv >= int (m_cellviews.size ())) {
    return;
  }

  QModelIndexList sel = lv_cells->selectionModel ()->selectedIndexes ();
  for (QModelIndexList::const_iterator s = sel.begin (); s != sel.end (); ++s) {
    db::CellGraph::cell_index_type ci = model->cell (*s)->cell_index ();
    mp_view->manager ()->transaction (tl::translate ("Show cells"));
    mp_view->show_cell (ci, m_current_cv);
    mp_view->manager ()->commit ();
  }

  model->signal_data_changed ();
}

void
CellSelectionFormImpl::hide_cell ()
{
  lay::CellTreeModel *model = dynamic_cast<lay::CellTreeModel *> (lv_cells->model ());
  if (! model) {
    return;
  }
  if (m_current_cv < 0 || m_current_cv >= int (m_cellviews.size ())) {
    return;
  }

  QModelIndexList sel = lv_cells->selectionModel ()->selectedIndexes ();
  for (QModelIndexList::const_iterator s = sel.begin (); s != sel.end (); ++s) {
    db::CellGraph::cell_index_type ci = model->cell (*s)->cell_index ();
    mp_view->manager ()->transaction (tl::translate ("Hide cells"));
    mp_view->hide_cell (ci, m_current_cv);
    mp_view->manager ()->commit ();
  }

  model->signal_data_changed ();
}
